# Translating AMPL model into pySCIPOpt

from pyscipopt import Model, quicksum, exp, log


def multiple_pts():

    model = Model("one_day_non_linear")    

  #### Variable Declaration
    q = {}
    for pt in PTs:
        for i in range(1,ttmax+1):
            q[pt,i] = model.addVar("q[%i,%i]"%(pt,i),ub=Qmax, lb=0) # Qmax probably needs to be lower, like 1.5. And the effects of surpassing rated load more severe.

    real_temp = {}
    for pt in PTs:
        for i in range(1,ttmax+1):
            real_temp[pt,i] = model.addVar("real_temp[%i,%i]"%(pt,i), lb=0)

    rul = {}
    for pt in PTs:
        for component in components:
            for t in range(0,ttmax+1):
                rul[pt,component.name,t] = model.addVar("rul[%i,%s,%i]"%(pt,component.name,t),lb=0,ub=component.RULmax)

    chi = {}
    for pt in PTs:
        for component in components:
            for t in range(0,Tmax+1):
                chi[pt,component.name,t] = model.addVar("chi[%i,%s,%i]"%(pt,component.name,t),vtype="B")


    #### Objective
    model.setObjective(quicksum(Price*q[pt,t] for pt in PTs for t in range(1,ttmax+1))-quicksum(k.cost*chi[pt,k.name,t] for pt in PTs for k in components for t in range(0,Tmax+1)), "maximize")

    ###############
    # Constraints #
    ###############
    
    #### RUL Constraints
    for pt in PTs:
        for k in components:
            model.addCons(rul[pt,k.name,0] == k.RULmax)

    for pt in PTs:
        for t in range(1,ttmax+1):     
            model.addCons(rul[pt,"Winding",t] <= rul[pt,"Winding",t-1] - exp(((real_temp[pt,t]-98)/6)*log(2)) + 5*Winding.RULmax*chi[pt,"Winding",t//(ttmax/Tmax)])
         
    #### Temperature
    for pt in PTs:
        for t in range(1,ttmax+1):
            model.addCons(real_temp[pt,t] >= A*exp(B*q[pt,t]))

    return model

Tmax  = 4
ttmax = Tmax*24
Price  = 1 # not used for load allocation
Qmax  = 2 
PTs = [i+1 for i in range(3)]

max_oil_cooling  = 5 

A  = 60 
B  = 0.5 


class Component:
    def __init__(self, name, RULmax, cost, maintenance_duration):
        """
        name:                   Component name as string
        RULmax:                 Component maximum Remaining Useful Life in years
        Cost:                   Component maintenance cost in nr of years to generate component cost 
        maintenance_duration:   Component duration of maintenance in nr of years (must be an integer)
        """

        self.name = name
        self.RULmax = RULmax*ttmax/Tmax
        self.cost = cost*ttmax/Tmax
        self.maintenance_duration = int(maintenance_duration*(ttmax/Tmax)*0)#maintenance_duration*0

Oil             = Component("Oil", 12, 0.2, 1)#/(12*24)) # 288, 4, 1)
Winding         = Component("Winding", 50, 12.5, 4)#/(12*24))#1200, 300, 4)

components = {Oil, Winding}

if __name__ == "__main__":
    model = multiple_pts()
    model.optimize()
